package com.logicbig.example;

import com.logicbig.uicommon.SuggestionDropDownDecorator;
import com.logicbig.uicommon.TextComponentSuggestionClient;
import com.logicbig.uicommon.TextComponentWordSuggestionClient;
import com.logicbig.util.RandomUtil;
import com.logicbig.util.TypingRobot2;
import javax.swing.*;
import java.awt.*;
import java.util.List;
import java.util.stream.Collectors;

public class SuggestionExampleMain {
    public static void main(String[] args) {
        JFrame frame = createFrame();
        JTextField textField = new JTextField(10);
        SuggestionDropDownDecorator.decorate(textField,
                new TextComponentSuggestionClient(SuggestionExampleMain::getSuggestions));
        JTextPane textPane = new JTextPane();
        SuggestionDropDownDecorator.decorate(textPane,
                new TextComponentWordSuggestionClient(SuggestionExampleMain::getSuggestions));
        frame.add(textField, BorderLayout.NORTH);
        frame.add(new JScrollPane(textPane));
        frame.setVisible(true);
    }

    private static List<String> words =
            RandomUtil.getWords(2, 400).stream().map(String::toLowerCase).collect(Collectors.toList());

    private static List<String> getSuggestions(String input) {
        //the suggestion provider can control text search related stuff, e.g case insensitive match, the search  limit etc.
        if (input.isEmpty()) {
            return null;
        }
        return words.stream()
                    .filter(s -> s.startsWith(input))
                    .limit(20)
                    .collect(Collectors.toList());
    }

    private static JFrame createFrame() {
        JFrame frame = new JFrame("Suggestion Dropdown Example");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(new Dimension(600, 300));
        return frame;
    }
}